package org.tbasket.dispatch

import zio.*
import zio.http.*
import zio.http.model.Status
import zio.http.model.Status.*
import zio.stream.ZStream

import java.nio.file.{Files, Path}
import scala.collection.mutable

class WebService(pagesLocation: Option[Path]) {
  
  private val resources = resolveResources
  
  
  def send(r: Request) =
    ZIO.attempt(pagesLocation.toString + r.url.path.toString)
      .filterOrFail(!_.startsWith("/"))(Forbidden)
      .map(resources.get)
      .someOrFail(NotFound)
      .map(content => Response(status = Ok, body = content))
      .catchSome {
        case status: Status =>
          ZIO.attempt(Response.status(status))
      }
  
  
  private def resolveResources: Map[String, Body] = {
    
    val map = mutable.HashMap.empty[String, Body]
    
    def resolveAll(loc: Path): Unit = {
      Files.list(loc)
        .forEach {
          case d if Files.isDirectory(d) => resolveAll(d)
          case f                         =>
            val body     = Body.fromFile(f.toFile)
            val fullPath = f.toString
            val fileName = f.getFileName.toString
            val purename = fileName.take(fileName.indexOf('.'))
            
            map.put(fullPath, body)
  
            //also bind the dir path with the index body
            if (purename == "index" || purename == f.getParent.getFileName.toString) {
              val extension = fullPath.drop(fullPath.indexOf('.'))
              val dirPath = f.getParent.toString
              if (extension == ".html")
                map.put(dirPath, body)
              else map.put(dirPath + extension, body)
            }
        }
    }
    pagesLocation.foreach(resolveAll)
    map.toMap
  }
}
