package org.tbasket.handler

import io.netty.handler.codec.http.QueryStringDecoder
import org.tbasket.error.*
import zio.http.Body
import zio.http.api.openapi.OpenAPI.Parameter.QueryStyle.Form
import zio.json.*
import zio.json.ast.{Json, JsonCursor}
import zio.{Task, ZIO}

import java.net.URLDecoder
import java.nio.charset.StandardCharsets
import scala.jdk.CollectionConverters.*
import scala.language.reflectiveCalls

object HandlerUtils {

  
  def errorBody(errorType: String, msg: String) = Body.fromString(s"""{"error": "$errorType","msg": "$msg"}""")
  
  def parseRequestForm(body: Body) =
    (for
      decoded <- body.asString.map(str => URLDecoder.decode(str, StandardCharsets.UTF_8.name()))
      params = decoded.split('&').collect { case s"$k=$v" => (k.stripPrefix("?"), v)}.toMap
    yield params)
      .mapError(s => InvalidRequest("Invalid request body", s.getMessage))
  
  def search(map: Map[String, String])(name: String) =
    ZIO.attempt(map.get(name)).someOrFail(InvalidRequest(s"Missing or invalid field $name."))
  
}